@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 '''
 @author: saaj
 '''
@@ -379,6 +380,36 @@ class TestGeneral(test.TestCase):
     self.assertTrue(response['error'] is None)
     self.assertEqual(90, response['result'])
   
+  def testUnicodeParams(self):
+    expected = [{'record' : u'Ænima'}, {'foo' : u'бар'}]
+    
+    self._post('echo', expected)
+    
+    self.assertStatus(200)
+    self.assertHeader('content-type', 'application/json')
+    
+    response = json.loads(self.body.decode())
+    self.assertTrue(response['error'] is None)
+    self.assertEqual(expected, response['result'])
+    
+    
+    # Unencoded utf-8 strings    
+    request = (u'{"params": [[{"record": "Ænima"}, {"foo": "бар"}]], ' 
+      u'"id": 1, "service": "qxcpjsonrpc.test.Service", "method": "echo"}').encode('utf-8')
+    headers = [
+      ('content-length', str(len(request))),
+      ('content-type',   'application/json; charset=utf-8'),
+    ]
+    
+    self.getPage('/service', method = 'post', body = request, headers = headers)
+    
+    self.assertStatus(200)
+    self.assertHeader('content-type', 'application/json')
+    
+    response = json.loads(self.body.decode())
+    self.assertTrue(response['error'] is None)
+    self.assertEqual(expected, response['result'])
+  
   def testInvalidHttpMethodHttpError(self):
     self.getPage('/service', method = 'options')
     
